@@ -21,20 +21,12 @@ themselves are kept in bin/.
 """
 
 __all__ = [
-    'RequestTokenApp',
-    'TrustedTokenAuthorizationConsoleApp',
-    ]
+    "RequestTokenApp",
+]
 
-import getpass
-import sys
+import json
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
-
-from launchpadlib.credentials import (
-    Credentials, RequestTokenAuthorizationEngine, TokenAuthorizationException)
+from launchpadlib.credentials import Credentials
 from launchpadlib.uris import lookup_web_root
 
 
@@ -50,8 +42,8 @@ class RequestTokenApp(object):
     def run(self):
         """Get a request token and return JSON information about it."""
         token = self.credentials.get_request_token(
-            self.context, self.web_root,
-            token_format=Credentials.DICT_TOKEN_FORMAT)
+            self.context,
+            self.web_root,
+            token_format=Credentials.DICT_TOKEN_FORMAT,
+        )
         return json.dumps(token)
-
-
