@@ -66,13 +66,16 @@ also in the WADL file itelf.
 """
 
 from datetime import datetime
-import collections
+
+try:
+    from collections.abc import Callable
+except ImportError:
+    from collections import Callable
 import sys
+
 if sys.version_info[0] >= 3:
     basestring = str
 
-JSON_MEDIA_TYPE = "application/json"
-
 
 class IntegrityError(Exception):
     """Raised when bad sample data is used with a L{FakeLaunchpad} instance."""
@@ -85,15 +88,27 @@ class FakeLaunchpad(object):
         Launchpad WADL definition file.
     """
 
-    def __init__(self, credentials=None, service_root=None, cache=None,
-                 timeout=None, proxy_info=None, application=None):
+    def __init__(
+        self,
+        credentials=None,
+        service_root=None,
+        cache=None,
+        timeout=None,
+        proxy_info=None,
+        application=None,
+    ):
         if application is None:
             from launchpadlib.testing.resources import get_application
+
             application = get_application()
         root_resource = FakeRoot(application)
-        self.__dict__.update({"credentials": credentials,
-                              "_application": application,
-                              "_service_root": root_resource})
+        self.__dict__.update(
+            {
+                "credentials": credentials,
+                "_application": application,
+                "_service_root": root_resource,
+            }
+        )
 
     def __setattr__(self, name, values):
         """Set sample data.
@@ -113,39 +128,66 @@ class FakeLaunchpad(object):
         return getattr(self._service_root, name)
 
     @classmethod
-    def login(cls, consumer_name, token_string, access_secret,
-              service_root=None, cache=None, timeout=None, proxy_info=None):
+    def login(
+        cls,
+        consumer_name,
+        token_string,
+        access_secret,
+        service_root=None,
+        cache=None,
+        timeout=None,
+        proxy_info=None,
+    ):
         """Convenience for setting up access credentials."""
         from launchpadlib.testing.resources import get_application
+
         return cls(object(), application=get_application())
 
     @classmethod
-    def get_token_and_login(cls, consumer_name, service_root=None,
-                            cache=None, timeout=None, proxy_info=None):
+    def get_token_and_login(
+        cls,
+        consumer_name,
+        service_root=None,
+        cache=None,
+        timeout=None,
+        proxy_info=None,
+    ):
         """Get credentials from Launchpad and log into the service root."""
         from launchpadlib.testing.resources import get_application
+
         return cls(object(), application=get_application())
 
     @classmethod
-    def login_with(cls, consumer_name, service_root=None,
-                   launchpadlib_dir=None, timeout=None, proxy_info=None):
+    def login_with(
+        cls,
+        consumer_name,
+        service_root=None,
+        launchpadlib_dir=None,
+        timeout=None,
+        proxy_info=None,
+    ):
         """Log in to Launchpad with possibly cached credentials."""
         from launchpadlib.testing.resources import get_application
+
         return cls(object(), application=get_application())
 
 
 def find_by_attribute(element, name, value):
-    """Find children of 'element' where attribute 'name' is equal to 'value'.
-    """
+    """Find children of element where attribute name is equal to value."""
     return [child for child in element if child.get(name) == value]
 
 
 def strip_suffix(string, suffix):
     if string.endswith(suffix):
-        return string[:-len(suffix)]
+        return string[: -len(suffix)]
     return string
 
 
+def wadl_tag(tag_name):
+    """Scope a tag name with the WADL namespace."""
+    return "{http://research.sun.com/wadl/2006/10}" + tag_name
+
+
 class FakeResource(object):
     """
     Represents valid sample data on L{FakeLaunchpad} instances.
@@ -173,10 +215,14 @@ class FakeResource(object):
         """
         if values is None:
             values = {}
-        self.__dict__.update({"_application": application,
-                              "_resource_type": resource_type,
-                              "_children": {},
-                              "_values": values})
+        self.__dict__.update(
+            {
+                "_application": application,
+                "_resource_type": resource_type,
+                "_children": {},
+                "_values": values,
+            }
+        )
 
     def __setattr__(self, name, value):
         """Set sample data.
@@ -205,7 +251,7 @@ class FakeResource(object):
         result = self._children.get(name, _marker)
         if result is _marker:
             result = self._values.get(name, _marker)
-            if isinstance(result, collections.Callable):
+            if isinstance(result, Callable):
                 return self._wrap_method(name, result)
         if name in self.special_methods:
             return lambda: True
@@ -219,8 +265,10 @@ class FakeResource(object):
         @param name: The name of the method.
         @param method: The callable to run when the method is called.
         """
+
         def wrapper(*args, **kwargs):
             return self._run_method(name, method, *args, **kwargs)
+
         return wrapper
 
     def _create_child_resource(self, name, values):
@@ -238,13 +286,17 @@ class FakeResource(object):
             this resource or if C{values} isn't a valid object for the C{name}
             attribute.
         """
-        root_resource = self._application.get_resource_by_path("")
+        xml_id = self._find_representation_id(self._resource_type, "get")
+        representation = self._application.representation_definitions[xml_id]
+        params = {
+            child.name: child
+            for child in representation.params(representation)
+        }
         is_link = False
-        param = root_resource.get_parameter(name + "_collection_link",
-                                            JSON_MEDIA_TYPE)
+        param = params.get(name + "_collection_link")
         if param is None:
             is_link = True
-            param = root_resource.get_parameter(name + "_link", JSON_MEDIA_TYPE)
+            param = params.get(name + "_link")
         if param is None:
             raise IntegrityError("%s isn't a valid property." % (name,))
         resource_type = self._get_resource_type(param)
@@ -252,10 +304,16 @@ class FakeResource(object):
             self._check_resource_type(resource_type, values)
             return FakeEntry(self._application, resource_type, values)
         else:
-            name, child_resource_type = (
-                self._check_collection_type(resource_type, values))
-            return FakeCollection(self._application, resource_type, values,
-                                  name, child_resource_type)
+            name, child_resource_type = self._check_collection_type(
+                resource_type, values
+            )
+            return FakeCollection(
+                self._application,
+                resource_type,
+                values,
+                name,
+                child_resource_type,
+            )
 
     def _get_resource_type(self, param):
         """Get the resource type for C{param}.
@@ -265,7 +323,7 @@ class FakeResource(object):
         @return: The resource type for the parameter, or None if one isn't
             available.
         """
-        [link] = list(param.tag)
+        link = param.tag.find(wadl_tag("link"))
         name = link.get("resource_type")
         return self._application.get_resource_type(name)
 
@@ -280,7 +338,7 @@ class FakeResource(object):
             attributes and methods.
         """
         for name, value in partial_object.items():
-            if isinstance(value, collections.Callable):
+            if isinstance(value, Callable):
                 # Performs an integrity check.
                 self._get_method(resource_type, name)
             else:
@@ -304,9 +362,10 @@ class FakeResource(object):
         child_resource_type = None
         for name, value in partial_object.items():
             if name == "entries":
-                name, child_resource_type = (
-                    self._check_entries(resource_type, value))
-            elif isinstance(value, collections.Callable):
+                name, child_resource_type = self._check_entries(
+                    resource_type, value
+                )
+            elif isinstance(value, Callable):
                 # Performs an integrity check.
                 self._get_method(resource_type, name)
             else:
@@ -340,7 +399,7 @@ class FakeResource(object):
         @param name: The name of the attribute.
         @param value: The value to check.
         """
-        xml_id = self._find_representation_id(resource_type, 'get')
+        xml_id = self._find_representation_id(resource_type, "get")
         self._check_attribute_representation(xml_id, name, value)
 
     def _check_attribute_representation(self, xml_id, name, value):
@@ -356,20 +415,34 @@ class FakeResource(object):
             name or if C{value}'s type is not valid for the attribute.
         """
         representation = self._application.representation_definitions[xml_id]
-        parameters = dict((child.get("name"), child)
-                          for child in representation.tag)
-        if name not in parameters:
-            raise IntegrityError("%s not found" % name)
-        parameter = parameters[name]
-        data_type = parameter.get("type")
-        if data_type is None:
-            if not isinstance(value, basestring):
-                raise IntegrityError(
-                    "%s is not a str or unicode for %s" % (value, name))
-        elif data_type == "xsd:dateTime":
-            if not isinstance(value, datetime):
-                raise IntegrityError(
-                    "%s is not a datetime for %s" % (value, name))
+        params = {
+            child.name: child
+            for child in representation.params(representation)
+        }
+        if name + "_collection_link" in params:
+            resource_type = self._get_resource_type(
+                params[name + "_collection_link"]
+            )
+            child_name, child_resource_type = self._check_collection_type(
+                resource_type, value
+            )
+        elif name + "_link" in params:
+            resource_type = self._get_resource_type(params[name + "_link"])
+            self._check_resource_type(resource_type, value)
+        else:
+            param = params.get(name)
+            if param is None:
+                raise IntegrityError("%s not found" % name)
+            if param.type is None:
+                if not isinstance(value, basestring):
+                    raise IntegrityError(
+                        "%s is not a str or unicode for %s" % (value, name)
+                    )
+            elif param.type == "xsd:dateTime":
+                if not isinstance(value, datetime):
+                    raise IntegrityError(
+                        "%s is not a datetime for %s" % (value, name)
+                    )
 
     def _get_method(self, resource_type, name):
         """Get the C{name} method on C{resource_type}.
@@ -385,10 +458,11 @@ class FakeResource(object):
         resource_name = resource_type.tag.get("id")
         xml_id = "%s-%s" % (resource_name, name)
         try:
-            [get_method] = find_by_attribute(resource_type.tag, 'id', xml_id)
+            [get_method] = find_by_attribute(resource_type.tag, "id", xml_id)
         except ValueError:
             raise IntegrityError(
-                "%s is not a method of %s" % (name, resource_name))
+                "%s is not a method of %s" % (name, resource_name)
+            )
         return get_method
 
     def _run_method(self, name, method, *args, **kwargs):
@@ -406,32 +480,49 @@ class FakeResource(object):
             isn't valid.
         """
         result = method(*args, **kwargs)
-        if name in self.special_methods:
+        if name in self.special_methods or result is None:
             return result
         else:
             return self._create_resource(self._resource_type, name, result)
 
     def _create_resource(self, resource_type, name, result):
-        """Create a new L{FakeResource} for C{resource_type} method call result.
+        """Create new L{FakeResource} for C{resource_type} method call result.
 
         @param resource_type: The resource type of the method.
         @param name: The name of the method on C{resource_type}.
         @param result: The result of calling the method.
         @raises IntegrityError: Raised if C{result} is an invalid return value
             for the method.
-        @return: A L{FakeResource} for C{result}.
+        @return: A L{FakeResource} for C{result}, or just C{result} if no
+            response representation is defined for the method.
         """
         resource_name = resource_type.tag.get("id")
         if resource_name == name:
             name = "get"
         xml_id = self._find_representation_id(resource_type, name)
-        xml_id = strip_suffix(xml_id, '-full')
+        if xml_id is None:
+            return result
+        xml_id = strip_suffix(xml_id, "-full")
         if xml_id not in self._application.resource_types:
-            xml_id += '-resource'
+            xml_id += "-resource"
         result_resource_type = self._application.resource_types[xml_id]
-        self._check_resource_type(result_resource_type, result)
-        # XXX: Should this wrap in collection?
-        return FakeResource(self._application, result_resource_type, result)
+        if xml_id.endswith("-page-resource"):
+            name, child_resource_type = self._check_collection_type(
+                result_resource_type, result
+            )
+            return FakeCollection(
+                self._application,
+                result_resource_type,
+                result,
+                name,
+                child_resource_type,
+            )
+        else:
+            self._check_resource_type(result_resource_type, result)
+            resource = FakeEntry(self._application, result_resource_type)
+            for child_name, child_value in result.items():
+                setattr(resource, child_name, child_value)
+            return resource
 
     def _get_child_resource_type(self, resource_type):
         """Get the name and resource type for the entries in a collection.
@@ -441,18 +532,21 @@ class FakeResource(object):
             resource type and 'resource_type' is the corresponding resource
             type.
         """
-        xml_id = self._find_representation_id(resource_type, 'get')
+        xml_id = self._find_representation_id(resource_type, "get")
         representation_definition = (
-            self._application.representation_definitions[xml_id])
+            self._application.representation_definitions[xml_id]
+        )
 
         [entry_links] = find_by_attribute(
-            representation_definition.tag, 'name', 'entry_links')
+            representation_definition.tag, "name", "entry_links"
+        )
         [resource_type] = list(entry_links)
         resource_type_url = resource_type.get("resource_type")
         resource_type_name = resource_type_url.split("#")[1]
         return (
             resource_type_name,
-            self._application.get_resource_type(resource_type_url))
+            self._application.get_resource_type(resource_type_url),
+        )
 
     def _check_entries(self, resource_type, entries):
         """Ensure that C{entries} are valid for a C{resource_type} collection.
@@ -465,7 +559,9 @@ class FakeResource(object):
             resource type and 'resource_type' is the corresponding resource
             type.
         """
-        name, child_resource_type = self._get_child_resource_type(resource_type)
+        name, child_resource_type = self._get_child_resource_type(
+            resource_type
+        )
         for entry in entries:
             self._check_resource_type(child_resource_type, entry)
         return name, child_resource_type
@@ -480,7 +576,11 @@ class FakeResource(object):
         key = self._values.get("id", key)
         key = self._values.get("name", key)
         return "<%s %s %s at %s>" % (
-            self.__class__.__name__, name, key, hex(id(self)))
+            self.__class__.__name__,
+            name,
+            key,
+            hex(id(self)),
+        )
 
 
 class FakeRoot(FakeResource):
@@ -492,7 +592,8 @@ class FakeRoot(FakeResource):
         @param application: A C{wadllib.application.Application} instance.
         """
         resource_type = application.get_resource_type(
-            application.markup_url + "#service-root")
+            application.markup_url + "#service-root"
+        )
         super(FakeRoot, self).__init__(application, resource_type)
 
 
@@ -503,18 +604,28 @@ class FakeEntry(FakeResource):
 class FakeCollection(FakeResource):
     """A fake resource for a collection."""
 
-    def __init__(self, application, resource_type, values=None,
-                 name=None, child_resource_type=None):
-        super(FakeCollection, self).__init__(application, resource_type, values)
-        self.__dict__.update({"_name": name,
-                              "_child_resource_type": child_resource_type})
+    def __init__(
+        self,
+        application,
+        resource_type,
+        values=None,
+        name=None,
+        child_resource_type=None,
+    ):
+        super(FakeCollection, self).__init__(
+            application, resource_type, values
+        )
+        self.__dict__.update(
+            {"_name": name, "_child_resource_type": child_resource_type}
+        )
 
     def __iter__(self):
         """Iterate items if this resource has an C{entries} attribute."""
         entries = self._values.get("entries", ())
         for entry in entries:
-            yield self._create_resource(self._child_resource_type, self._name,
-                                        entry)
+            yield self._create_resource(
+                self._child_resource_type, self._name, entry
+            )
 
     def __getitem__(self, key):
         """Look up a slice, or a subordinate resource by index.
@@ -528,11 +639,14 @@ class FakeCollection(FakeResource):
             start = key.start or 0
             stop = key.stop
             if start < 0:
-                raise ValueError("Collection slices must have a nonnegative "
-                                 "start point.")
+                raise ValueError(
+                    "Collection slices must have a nonnegative " "start point."
+                )
             if stop < 0:
-                raise ValueError("Collection slices must have a definite, "
-                                 "nonnegative end point.")
+                raise ValueError(
+                    "Collection slices must have a definite, "
+                    "nonnegative end point."
+                )
             return entries.__getitem__(key)
         elif isinstance(key, int):
             return entries.__getitem__(key)
